# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class RepeatSchedule(core.Gs2Model):
    repeat_count: int = None
    current_repeat_start_at: int = None
    current_repeat_end_at: int = None
    last_repeat_end_at: int = None
    next_repeat_start_at: int = None

    def with_repeat_count(self, repeat_count: int) -> RepeatSchedule:
        self.repeat_count = repeat_count
        return self

    def with_current_repeat_start_at(self, current_repeat_start_at: int) -> RepeatSchedule:
        self.current_repeat_start_at = current_repeat_start_at
        return self

    def with_current_repeat_end_at(self, current_repeat_end_at: int) -> RepeatSchedule:
        self.current_repeat_end_at = current_repeat_end_at
        return self

    def with_last_repeat_end_at(self, last_repeat_end_at: int) -> RepeatSchedule:
        self.last_repeat_end_at = last_repeat_end_at
        return self

    def with_next_repeat_start_at(self, next_repeat_start_at: int) -> RepeatSchedule:
        self.next_repeat_start_at = next_repeat_start_at
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RepeatSchedule]:
        if data is None:
            return None
        return RepeatSchedule()\
            .with_repeat_count(data.get('repeatCount'))\
            .with_current_repeat_start_at(data.get('currentRepeatStartAt'))\
            .with_current_repeat_end_at(data.get('currentRepeatEndAt'))\
            .with_last_repeat_end_at(data.get('lastRepeatEndAt'))\
            .with_next_repeat_start_at(data.get('nextRepeatStartAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "repeatCount": self.repeat_count,
            "currentRepeatStartAt": self.current_repeat_start_at,
            "currentRepeatEndAt": self.current_repeat_end_at,
            "lastRepeatEndAt": self.last_repeat_end_at,
            "nextRepeatStartAt": self.next_repeat_start_at,
        }


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class GitHubCheckoutSetting(core.Gs2Model):
    api_key_id: str = None
    repository_name: str = None
    source_path: str = None
    reference_type: str = None
    commit_hash: str = None
    branch_name: str = None
    tag_name: str = None

    def with_api_key_id(self, api_key_id: str) -> GitHubCheckoutSetting:
        self.api_key_id = api_key_id
        return self

    def with_repository_name(self, repository_name: str) -> GitHubCheckoutSetting:
        self.repository_name = repository_name
        return self

    def with_source_path(self, source_path: str) -> GitHubCheckoutSetting:
        self.source_path = source_path
        return self

    def with_reference_type(self, reference_type: str) -> GitHubCheckoutSetting:
        self.reference_type = reference_type
        return self

    def with_commit_hash(self, commit_hash: str) -> GitHubCheckoutSetting:
        self.commit_hash = commit_hash
        return self

    def with_branch_name(self, branch_name: str) -> GitHubCheckoutSetting:
        self.branch_name = branch_name
        return self

    def with_tag_name(self, tag_name: str) -> GitHubCheckoutSetting:
        self.tag_name = tag_name
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GitHubCheckoutSetting]:
        if data is None:
            return None
        return GitHubCheckoutSetting()\
            .with_api_key_id(data.get('apiKeyId'))\
            .with_repository_name(data.get('repositoryName'))\
            .with_source_path(data.get('sourcePath'))\
            .with_reference_type(data.get('referenceType'))\
            .with_commit_hash(data.get('commitHash'))\
            .with_branch_name(data.get('branchName'))\
            .with_tag_name(data.get('tagName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "apiKeyId": self.api_key_id,
            "repositoryName": self.repository_name,
            "sourcePath": self.source_path,
            "referenceType": self.reference_type,
            "commitHash": self.commit_hash,
            "branchName": self.branch_name,
            "tagName": self.tag_name,
        }


class CurrentEventMaster(core.Gs2Model):
    namespace_id: str = None
    settings: str = None

    def with_namespace_id(self, namespace_id: str) -> CurrentEventMaster:
        self.namespace_id = namespace_id
        return self

    def with_settings(self, settings: str) -> CurrentEventMaster:
        self.settings = settings
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:schedule:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CurrentEventMaster]:
        if data is None:
            return None
        return CurrentEventMaster()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_settings(data.get('settings'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "settings": self.settings,
        }


class Event(core.Gs2Model):
    event_id: str = None
    name: str = None
    metadata: str = None
    schedule_type: str = None
    repeat_type: str = None
    absolute_begin: int = None
    absolute_end: int = None
    repeat_begin_day_of_month: int = None
    repeat_end_day_of_month: int = None
    repeat_begin_day_of_week: str = None
    repeat_end_day_of_week: str = None
    repeat_begin_hour: int = None
    repeat_end_hour: int = None
    relative_trigger_name: str = None

    def with_event_id(self, event_id: str) -> Event:
        self.event_id = event_id
        return self

    def with_name(self, name: str) -> Event:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> Event:
        self.metadata = metadata
        return self

    def with_schedule_type(self, schedule_type: str) -> Event:
        self.schedule_type = schedule_type
        return self

    def with_repeat_type(self, repeat_type: str) -> Event:
        self.repeat_type = repeat_type
        return self

    def with_absolute_begin(self, absolute_begin: int) -> Event:
        self.absolute_begin = absolute_begin
        return self

    def with_absolute_end(self, absolute_end: int) -> Event:
        self.absolute_end = absolute_end
        return self

    def with_repeat_begin_day_of_month(self, repeat_begin_day_of_month: int) -> Event:
        self.repeat_begin_day_of_month = repeat_begin_day_of_month
        return self

    def with_repeat_end_day_of_month(self, repeat_end_day_of_month: int) -> Event:
        self.repeat_end_day_of_month = repeat_end_day_of_month
        return self

    def with_repeat_begin_day_of_week(self, repeat_begin_day_of_week: str) -> Event:
        self.repeat_begin_day_of_week = repeat_begin_day_of_week
        return self

    def with_repeat_end_day_of_week(self, repeat_end_day_of_week: str) -> Event:
        self.repeat_end_day_of_week = repeat_end_day_of_week
        return self

    def with_repeat_begin_hour(self, repeat_begin_hour: int) -> Event:
        self.repeat_begin_hour = repeat_begin_hour
        return self

    def with_repeat_end_hour(self, repeat_end_hour: int) -> Event:
        self.repeat_end_hour = repeat_end_hour
        return self

    def with_relative_trigger_name(self, relative_trigger_name: str) -> Event:
        self.relative_trigger_name = relative_trigger_name
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        event_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:schedule:{namespaceName}:event:{eventName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            eventName=event_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_event_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('event_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Event]:
        if data is None:
            return None
        return Event()\
            .with_event_id(data.get('eventId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_schedule_type(data.get('scheduleType'))\
            .with_repeat_type(data.get('repeatType'))\
            .with_absolute_begin(data.get('absoluteBegin'))\
            .with_absolute_end(data.get('absoluteEnd'))\
            .with_repeat_begin_day_of_month(data.get('repeatBeginDayOfMonth'))\
            .with_repeat_end_day_of_month(data.get('repeatEndDayOfMonth'))\
            .with_repeat_begin_day_of_week(data.get('repeatBeginDayOfWeek'))\
            .with_repeat_end_day_of_week(data.get('repeatEndDayOfWeek'))\
            .with_repeat_begin_hour(data.get('repeatBeginHour'))\
            .with_repeat_end_hour(data.get('repeatEndHour'))\
            .with_relative_trigger_name(data.get('relativeTriggerName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "eventId": self.event_id,
            "name": self.name,
            "metadata": self.metadata,
            "scheduleType": self.schedule_type,
            "repeatType": self.repeat_type,
            "absoluteBegin": self.absolute_begin,
            "absoluteEnd": self.absolute_end,
            "repeatBeginDayOfMonth": self.repeat_begin_day_of_month,
            "repeatEndDayOfMonth": self.repeat_end_day_of_month,
            "repeatBeginDayOfWeek": self.repeat_begin_day_of_week,
            "repeatEndDayOfWeek": self.repeat_end_day_of_week,
            "repeatBeginHour": self.repeat_begin_hour,
            "repeatEndHour": self.repeat_end_hour,
            "relativeTriggerName": self.relative_trigger_name,
        }


class Trigger(core.Gs2Model):
    trigger_id: str = None
    name: str = None
    user_id: str = None
    created_at: int = None
    expires_at: int = None
    revision: int = None

    def with_trigger_id(self, trigger_id: str) -> Trigger:
        self.trigger_id = trigger_id
        return self

    def with_name(self, name: str) -> Trigger:
        self.name = name
        return self

    def with_user_id(self, user_id: str) -> Trigger:
        self.user_id = user_id
        return self

    def with_created_at(self, created_at: int) -> Trigger:
        self.created_at = created_at
        return self

    def with_expires_at(self, expires_at: int) -> Trigger:
        self.expires_at = expires_at
        return self

    def with_revision(self, revision: int) -> Trigger:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        trigger_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:schedule:{namespaceName}:user:{userId}:trigger:{triggerName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            triggerName=trigger_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):user:(?P<userId>.+):trigger:(?P<triggerName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):user:(?P<userId>.+):trigger:(?P<triggerName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):user:(?P<userId>.+):trigger:(?P<triggerName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):user:(?P<userId>.+):trigger:(?P<triggerName>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_trigger_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):user:(?P<userId>.+):trigger:(?P<triggerName>.+)', grn)
        if match is None:
            return None
        return match.group('trigger_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Trigger]:
        if data is None:
            return None
        return Trigger()\
            .with_trigger_id(data.get('triggerId'))\
            .with_name(data.get('name'))\
            .with_user_id(data.get('userId'))\
            .with_created_at(data.get('createdAt'))\
            .with_expires_at(data.get('expiresAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "triggerId": self.trigger_id,
            "name": self.name,
            "userId": self.user_id,
            "createdAt": self.created_at,
            "expiresAt": self.expires_at,
            "revision": self.revision,
        }


class EventMaster(core.Gs2Model):
    event_id: str = None
    name: str = None
    description: str = None
    metadata: str = None
    schedule_type: str = None
    repeat_type: str = None
    absolute_begin: int = None
    absolute_end: int = None
    repeat_begin_day_of_month: int = None
    repeat_end_day_of_month: int = None
    repeat_begin_day_of_week: str = None
    repeat_end_day_of_week: str = None
    repeat_begin_hour: int = None
    repeat_end_hour: int = None
    relative_trigger_name: str = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_event_id(self, event_id: str) -> EventMaster:
        self.event_id = event_id
        return self

    def with_name(self, name: str) -> EventMaster:
        self.name = name
        return self

    def with_description(self, description: str) -> EventMaster:
        self.description = description
        return self

    def with_metadata(self, metadata: str) -> EventMaster:
        self.metadata = metadata
        return self

    def with_schedule_type(self, schedule_type: str) -> EventMaster:
        self.schedule_type = schedule_type
        return self

    def with_repeat_type(self, repeat_type: str) -> EventMaster:
        self.repeat_type = repeat_type
        return self

    def with_absolute_begin(self, absolute_begin: int) -> EventMaster:
        self.absolute_begin = absolute_begin
        return self

    def with_absolute_end(self, absolute_end: int) -> EventMaster:
        self.absolute_end = absolute_end
        return self

    def with_repeat_begin_day_of_month(self, repeat_begin_day_of_month: int) -> EventMaster:
        self.repeat_begin_day_of_month = repeat_begin_day_of_month
        return self

    def with_repeat_end_day_of_month(self, repeat_end_day_of_month: int) -> EventMaster:
        self.repeat_end_day_of_month = repeat_end_day_of_month
        return self

    def with_repeat_begin_day_of_week(self, repeat_begin_day_of_week: str) -> EventMaster:
        self.repeat_begin_day_of_week = repeat_begin_day_of_week
        return self

    def with_repeat_end_day_of_week(self, repeat_end_day_of_week: str) -> EventMaster:
        self.repeat_end_day_of_week = repeat_end_day_of_week
        return self

    def with_repeat_begin_hour(self, repeat_begin_hour: int) -> EventMaster:
        self.repeat_begin_hour = repeat_begin_hour
        return self

    def with_repeat_end_hour(self, repeat_end_hour: int) -> EventMaster:
        self.repeat_end_hour = repeat_end_hour
        return self

    def with_relative_trigger_name(self, relative_trigger_name: str) -> EventMaster:
        self.relative_trigger_name = relative_trigger_name
        return self

    def with_created_at(self, created_at: int) -> EventMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> EventMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> EventMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        event_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:schedule:{namespaceName}:event:{eventName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            eventName=event_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_event_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+):event:(?P<eventName>.+)', grn)
        if match is None:
            return None
        return match.group('event_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[EventMaster]:
        if data is None:
            return None
        return EventMaster()\
            .with_event_id(data.get('eventId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_metadata(data.get('metadata'))\
            .with_schedule_type(data.get('scheduleType'))\
            .with_repeat_type(data.get('repeatType'))\
            .with_absolute_begin(data.get('absoluteBegin'))\
            .with_absolute_end(data.get('absoluteEnd'))\
            .with_repeat_begin_day_of_month(data.get('repeatBeginDayOfMonth'))\
            .with_repeat_end_day_of_month(data.get('repeatEndDayOfMonth'))\
            .with_repeat_begin_day_of_week(data.get('repeatBeginDayOfWeek'))\
            .with_repeat_end_day_of_week(data.get('repeatEndDayOfWeek'))\
            .with_repeat_begin_hour(data.get('repeatBeginHour'))\
            .with_repeat_end_hour(data.get('repeatEndHour'))\
            .with_relative_trigger_name(data.get('relativeTriggerName'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "eventId": self.event_id,
            "name": self.name,
            "description": self.description,
            "metadata": self.metadata,
            "scheduleType": self.schedule_type,
            "repeatType": self.repeat_type,
            "absoluteBegin": self.absolute_begin,
            "absoluteEnd": self.absolute_end,
            "repeatBeginDayOfMonth": self.repeat_begin_day_of_month,
            "repeatEndDayOfMonth": self.repeat_end_day_of_month,
            "repeatBeginDayOfWeek": self.repeat_begin_day_of_week,
            "repeatEndDayOfWeek": self.repeat_end_day_of_week,
            "repeatBeginHour": self.repeat_begin_hour,
            "repeatEndHour": self.repeat_end_hour,
            "relativeTriggerName": self.relative_trigger_name,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:schedule:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):schedule:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class GitHubCheckoutSetting(core.Gs2Model):
    api_key_id: str = None
    repository_name: str = None
    source_path: str = None
    reference_type: str = None
    commit_hash: str = None
    branch_name: str = None
    tag_name: str = None

    def with_api_key_id(self, api_key_id: str) -> GitHubCheckoutSetting:
        self.api_key_id = api_key_id
        return self

    def with_repository_name(self, repository_name: str) -> GitHubCheckoutSetting:
        self.repository_name = repository_name
        return self

    def with_source_path(self, source_path: str) -> GitHubCheckoutSetting:
        self.source_path = source_path
        return self

    def with_reference_type(self, reference_type: str) -> GitHubCheckoutSetting:
        self.reference_type = reference_type
        return self

    def with_commit_hash(self, commit_hash: str) -> GitHubCheckoutSetting:
        self.commit_hash = commit_hash
        return self

    def with_branch_name(self, branch_name: str) -> GitHubCheckoutSetting:
        self.branch_name = branch_name
        return self

    def with_tag_name(self, tag_name: str) -> GitHubCheckoutSetting:
        self.tag_name = tag_name
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GitHubCheckoutSetting]:
        if data is None:
            return None
        return GitHubCheckoutSetting()\
            .with_api_key_id(data.get('apiKeyId'))\
            .with_repository_name(data.get('repositoryName'))\
            .with_source_path(data.get('sourcePath'))\
            .with_reference_type(data.get('referenceType'))\
            .with_commit_hash(data.get('commitHash'))\
            .with_branch_name(data.get('branchName'))\
            .with_tag_name(data.get('tagName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "apiKeyId": self.api_key_id,
            "repositoryName": self.repository_name,
            "sourcePath": self.source_path,
            "referenceType": self.reference_type,
            "commitHash": self.commit_hash,
            "branchName": self.branch_name,
            "tagName": self.tag_name,
        }


class SubscribeUser(core.Gs2Model):
    subscribe_user_id: str = None
    category_name: str = None
    user_id: str = None
    target_user_id: str = None

    def with_subscribe_user_id(self, subscribe_user_id: str) -> SubscribeUser:
        self.subscribe_user_id = subscribe_user_id
        return self

    def with_category_name(self, category_name: str) -> SubscribeUser:
        self.category_name = category_name
        return self

    def with_user_id(self, user_id: str) -> SubscribeUser:
        self.user_id = user_id
        return self

    def with_target_user_id(self, target_user_id: str) -> SubscribeUser:
        self.target_user_id = target_user_id
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        category_name,
        target_user_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}:user:{userId}:subscribe:category:{categoryName}:{targetUserId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            categoryName=category_name,
            targetUserId=target_user_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_category_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('category_name')

    @classmethod
    def get_target_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+):(?P<targetUserId>.+)', grn)
        if match is None:
            return None
        return match.group('target_user_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[SubscribeUser]:
        if data is None:
            return None
        return SubscribeUser()\
            .with_subscribe_user_id(data.get('subscribeUserId'))\
            .with_category_name(data.get('categoryName'))\
            .with_user_id(data.get('userId'))\
            .with_target_user_id(data.get('targetUserId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "subscribeUserId": self.subscribe_user_id,
            "categoryName": self.category_name,
            "userId": self.user_id,
            "targetUserId": self.target_user_id,
        }


class CalculatedAt(core.Gs2Model):
    category_name: str = None
    calculated_at: int = None

    def with_category_name(self, category_name: str) -> CalculatedAt:
        self.category_name = category_name
        return self

    def with_calculated_at(self, calculated_at: int) -> CalculatedAt:
        self.calculated_at = calculated_at
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CalculatedAt]:
        if data is None:
            return None
        return CalculatedAt()\
            .with_category_name(data.get('categoryName'))\
            .with_calculated_at(data.get('calculatedAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "categoryName": self.category_name,
            "calculatedAt": self.calculated_at,
        }


class CurrentRankingMaster(core.Gs2Model):
    namespace_id: str = None
    settings: str = None

    def with_namespace_id(self, namespace_id: str) -> CurrentRankingMaster:
        self.namespace_id = namespace_id
        return self

    def with_settings(self, settings: str) -> CurrentRankingMaster:
        self.settings = settings
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CurrentRankingMaster]:
        if data is None:
            return None
        return CurrentRankingMaster()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_settings(data.get('settings'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "settings": self.settings,
        }


class Scope(core.Gs2Model):
    name: str = None
    target_days: int = None

    def with_name(self, name: str) -> Scope:
        self.name = name
        return self

    def with_target_days(self, target_days: int) -> Scope:
        self.target_days = target_days
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Scope]:
        if data is None:
            return None
        return Scope()\
            .with_name(data.get('name'))\
            .with_target_days(data.get('targetDays'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "targetDays": self.target_days,
        }


class Ranking(core.Gs2Model):
    rank: int = None
    index: int = None
    category_name: str = None
    user_id: str = None
    score: int = None
    metadata: str = None
    created_at: int = None

    def with_rank(self, rank: int) -> Ranking:
        self.rank = rank
        return self

    def with_index(self, index: int) -> Ranking:
        self.index = index
        return self

    def with_category_name(self, category_name: str) -> Ranking:
        self.category_name = category_name
        return self

    def with_user_id(self, user_id: str) -> Ranking:
        self.user_id = user_id
        return self

    def with_score(self, score: int) -> Ranking:
        self.score = score
        return self

    def with_metadata(self, metadata: str) -> Ranking:
        self.metadata = metadata
        return self

    def with_created_at(self, created_at: int) -> Ranking:
        self.created_at = created_at
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Ranking]:
        if data is None:
            return None
        return Ranking()\
            .with_rank(data.get('rank'))\
            .with_index(data.get('index'))\
            .with_category_name(data.get('categoryName'))\
            .with_user_id(data.get('userId'))\
            .with_score(data.get('score'))\
            .with_metadata(data.get('metadata'))\
            .with_created_at(data.get('createdAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "rank": self.rank,
            "index": self.index,
            "categoryName": self.category_name,
            "userId": self.user_id,
            "score": self.score,
            "metadata": self.metadata,
            "createdAt": self.created_at,
        }


class Score(core.Gs2Model):
    score_id: str = None
    category_name: str = None
    user_id: str = None
    unique_id: str = None
    scorer_user_id: str = None
    score: int = None
    metadata: str = None
    created_at: int = None

    def with_score_id(self, score_id: str) -> Score:
        self.score_id = score_id
        return self

    def with_category_name(self, category_name: str) -> Score:
        self.category_name = category_name
        return self

    def with_user_id(self, user_id: str) -> Score:
        self.user_id = user_id
        return self

    def with_unique_id(self, unique_id: str) -> Score:
        self.unique_id = unique_id
        return self

    def with_scorer_user_id(self, scorer_user_id: str) -> Score:
        self.scorer_user_id = scorer_user_id
        return self

    def with_score(self, score: int) -> Score:
        self.score = score
        return self

    def with_metadata(self, metadata: str) -> Score:
        self.metadata = metadata
        return self

    def with_created_at(self, created_at: int) -> Score:
        self.created_at = created_at
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        category_name,
        scorer_user_id,
        unique_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}:user:{userId}:category:{categoryName}:score:{scorerUserId}:{uniqueId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            categoryName=category_name,
            scorerUserId=scorer_user_id,
            uniqueId=unique_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_category_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('category_name')

    @classmethod
    def get_scorer_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('scorer_user_id')

    @classmethod
    def get_unique_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):category:(?P<categoryName>.+):score:(?P<scorerUserId>.+):(?P<uniqueId>.+)', grn)
        if match is None:
            return None
        return match.group('unique_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Score]:
        if data is None:
            return None
        return Score()\
            .with_score_id(data.get('scoreId'))\
            .with_category_name(data.get('categoryName'))\
            .with_user_id(data.get('userId'))\
            .with_unique_id(data.get('uniqueId'))\
            .with_scorer_user_id(data.get('scorerUserId'))\
            .with_score(data.get('score'))\
            .with_metadata(data.get('metadata'))\
            .with_created_at(data.get('createdAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "scoreId": self.score_id,
            "categoryName": self.category_name,
            "userId": self.user_id,
            "uniqueId": self.unique_id,
            "scorerUserId": self.scorer_user_id,
            "score": self.score,
            "metadata": self.metadata,
            "createdAt": self.created_at,
        }


class Subscribe(core.Gs2Model):
    subscribe_id: str = None
    category_name: str = None
    user_id: str = None
    target_user_ids: List[str] = None
    subscribed_user_ids: List[str] = None
    created_at: int = None
    revision: int = None

    def with_subscribe_id(self, subscribe_id: str) -> Subscribe:
        self.subscribe_id = subscribe_id
        return self

    def with_category_name(self, category_name: str) -> Subscribe:
        self.category_name = category_name
        return self

    def with_user_id(self, user_id: str) -> Subscribe:
        self.user_id = user_id
        return self

    def with_target_user_ids(self, target_user_ids: List[str]) -> Subscribe:
        self.target_user_ids = target_user_ids
        return self

    def with_subscribed_user_ids(self, subscribed_user_ids: List[str]) -> Subscribe:
        self.subscribed_user_ids = subscribed_user_ids
        return self

    def with_created_at(self, created_at: int) -> Subscribe:
        self.created_at = created_at
        return self

    def with_revision(self, revision: int) -> Subscribe:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        category_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}:user:{userId}:subscribe:category:{categoryName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            categoryName=category_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_category_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):user:(?P<userId>.+):subscribe:category:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('category_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Subscribe]:
        if data is None:
            return None
        return Subscribe()\
            .with_subscribe_id(data.get('subscribeId'))\
            .with_category_name(data.get('categoryName'))\
            .with_user_id(data.get('userId'))\
            .with_target_user_ids([
                data.get('targetUserIds')[i]
                for i in range(len(data.get('targetUserIds')) if data.get('targetUserIds') else 0)
            ])\
            .with_subscribed_user_ids([
                data.get('subscribedUserIds')[i]
                for i in range(len(data.get('subscribedUserIds')) if data.get('subscribedUserIds') else 0)
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "subscribeId": self.subscribe_id,
            "categoryName": self.category_name,
            "userId": self.user_id,
            "targetUserIds": [
                self.target_user_ids[i]
                for i in range(len(self.target_user_ids) if self.target_user_ids else 0)
            ],
            "subscribedUserIds": [
                self.subscribed_user_ids[i]
                for i in range(len(self.subscribed_user_ids) if self.subscribed_user_ids else 0)
            ],
            "createdAt": self.created_at,
            "revision": self.revision,
        }


class CategoryModelMaster(core.Gs2Model):
    category_model_id: str = None
    name: str = None
    description: str = None
    metadata: str = None
    minimum_value: int = None
    maximum_value: int = None
    order_direction: str = None
    scope: str = None
    unique_by_user_id: bool = None
    sum: bool = None
    calculate_fixed_timing_hour: int = None
    calculate_fixed_timing_minute: int = None
    calculate_interval_minutes: int = None
    additional_scopes: List[Scope] = None
    entry_period_event_id: str = None
    access_period_event_id: str = None
    ignore_user_ids: List[str] = None
    generation: str = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_category_model_id(self, category_model_id: str) -> CategoryModelMaster:
        self.category_model_id = category_model_id
        return self

    def with_name(self, name: str) -> CategoryModelMaster:
        self.name = name
        return self

    def with_description(self, description: str) -> CategoryModelMaster:
        self.description = description
        return self

    def with_metadata(self, metadata: str) -> CategoryModelMaster:
        self.metadata = metadata
        return self

    def with_minimum_value(self, minimum_value: int) -> CategoryModelMaster:
        self.minimum_value = minimum_value
        return self

    def with_maximum_value(self, maximum_value: int) -> CategoryModelMaster:
        self.maximum_value = maximum_value
        return self

    def with_order_direction(self, order_direction: str) -> CategoryModelMaster:
        self.order_direction = order_direction
        return self

    def with_scope(self, scope: str) -> CategoryModelMaster:
        self.scope = scope
        return self

    def with_unique_by_user_id(self, unique_by_user_id: bool) -> CategoryModelMaster:
        self.unique_by_user_id = unique_by_user_id
        return self

    def with_sum(self, sum: bool) -> CategoryModelMaster:
        self.sum = sum
        return self

    def with_calculate_fixed_timing_hour(self, calculate_fixed_timing_hour: int) -> CategoryModelMaster:
        self.calculate_fixed_timing_hour = calculate_fixed_timing_hour
        return self

    def with_calculate_fixed_timing_minute(self, calculate_fixed_timing_minute: int) -> CategoryModelMaster:
        self.calculate_fixed_timing_minute = calculate_fixed_timing_minute
        return self

    def with_calculate_interval_minutes(self, calculate_interval_minutes: int) -> CategoryModelMaster:
        self.calculate_interval_minutes = calculate_interval_minutes
        return self

    def with_additional_scopes(self, additional_scopes: List[Scope]) -> CategoryModelMaster:
        self.additional_scopes = additional_scopes
        return self

    def with_entry_period_event_id(self, entry_period_event_id: str) -> CategoryModelMaster:
        self.entry_period_event_id = entry_period_event_id
        return self

    def with_access_period_event_id(self, access_period_event_id: str) -> CategoryModelMaster:
        self.access_period_event_id = access_period_event_id
        return self

    def with_ignore_user_ids(self, ignore_user_ids: List[str]) -> CategoryModelMaster:
        self.ignore_user_ids = ignore_user_ids
        return self

    def with_generation(self, generation: str) -> CategoryModelMaster:
        self.generation = generation
        return self

    def with_created_at(self, created_at: int) -> CategoryModelMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> CategoryModelMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> CategoryModelMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        category_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}:categoryModelMaster:{categoryName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            categoryName=category_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModelMaster:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModelMaster:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModelMaster:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_category_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModelMaster:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('category_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CategoryModelMaster]:
        if data is None:
            return None
        return CategoryModelMaster()\
            .with_category_model_id(data.get('categoryModelId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_metadata(data.get('metadata'))\
            .with_minimum_value(data.get('minimumValue'))\
            .with_maximum_value(data.get('maximumValue'))\
            .with_order_direction(data.get('orderDirection'))\
            .with_scope(data.get('scope'))\
            .with_unique_by_user_id(data.get('uniqueByUserId'))\
            .with_sum(data.get('sum'))\
            .with_calculate_fixed_timing_hour(data.get('calculateFixedTimingHour'))\
            .with_calculate_fixed_timing_minute(data.get('calculateFixedTimingMinute'))\
            .with_calculate_interval_minutes(data.get('calculateIntervalMinutes'))\
            .with_additional_scopes([
                Scope.from_dict(data.get('additionalScopes')[i])
                for i in range(len(data.get('additionalScopes')) if data.get('additionalScopes') else 0)
            ])\
            .with_entry_period_event_id(data.get('entryPeriodEventId'))\
            .with_access_period_event_id(data.get('accessPeriodEventId'))\
            .with_ignore_user_ids([
                data.get('ignoreUserIds')[i]
                for i in range(len(data.get('ignoreUserIds')) if data.get('ignoreUserIds') else 0)
            ])\
            .with_generation(data.get('generation'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "categoryModelId": self.category_model_id,
            "name": self.name,
            "description": self.description,
            "metadata": self.metadata,
            "minimumValue": self.minimum_value,
            "maximumValue": self.maximum_value,
            "orderDirection": self.order_direction,
            "scope": self.scope,
            "uniqueByUserId": self.unique_by_user_id,
            "sum": self.sum,
            "calculateFixedTimingHour": self.calculate_fixed_timing_hour,
            "calculateFixedTimingMinute": self.calculate_fixed_timing_minute,
            "calculateIntervalMinutes": self.calculate_interval_minutes,
            "additionalScopes": [
                self.additional_scopes[i].to_dict() if self.additional_scopes[i] else None
                for i in range(len(self.additional_scopes) if self.additional_scopes else 0)
            ],
            "entryPeriodEventId": self.entry_period_event_id,
            "accessPeriodEventId": self.access_period_event_id,
            "ignoreUserIds": [
                self.ignore_user_ids[i]
                for i in range(len(self.ignore_user_ids) if self.ignore_user_ids else 0)
            ],
            "generation": self.generation,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class CategoryModel(core.Gs2Model):
    category_model_id: str = None
    name: str = None
    metadata: str = None
    minimum_value: int = None
    maximum_value: int = None
    order_direction: str = None
    scope: str = None
    unique_by_user_id: bool = None
    sum: bool = None
    calculate_fixed_timing_hour: int = None
    calculate_fixed_timing_minute: int = None
    calculate_interval_minutes: int = None
    additional_scopes: List[Scope] = None
    entry_period_event_id: str = None
    access_period_event_id: str = None
    ignore_user_ids: List[str] = None
    generation: str = None

    def with_category_model_id(self, category_model_id: str) -> CategoryModel:
        self.category_model_id = category_model_id
        return self

    def with_name(self, name: str) -> CategoryModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> CategoryModel:
        self.metadata = metadata
        return self

    def with_minimum_value(self, minimum_value: int) -> CategoryModel:
        self.minimum_value = minimum_value
        return self

    def with_maximum_value(self, maximum_value: int) -> CategoryModel:
        self.maximum_value = maximum_value
        return self

    def with_order_direction(self, order_direction: str) -> CategoryModel:
        self.order_direction = order_direction
        return self

    def with_scope(self, scope: str) -> CategoryModel:
        self.scope = scope
        return self

    def with_unique_by_user_id(self, unique_by_user_id: bool) -> CategoryModel:
        self.unique_by_user_id = unique_by_user_id
        return self

    def with_sum(self, sum: bool) -> CategoryModel:
        self.sum = sum
        return self

    def with_calculate_fixed_timing_hour(self, calculate_fixed_timing_hour: int) -> CategoryModel:
        self.calculate_fixed_timing_hour = calculate_fixed_timing_hour
        return self

    def with_calculate_fixed_timing_minute(self, calculate_fixed_timing_minute: int) -> CategoryModel:
        self.calculate_fixed_timing_minute = calculate_fixed_timing_minute
        return self

    def with_calculate_interval_minutes(self, calculate_interval_minutes: int) -> CategoryModel:
        self.calculate_interval_minutes = calculate_interval_minutes
        return self

    def with_additional_scopes(self, additional_scopes: List[Scope]) -> CategoryModel:
        self.additional_scopes = additional_scopes
        return self

    def with_entry_period_event_id(self, entry_period_event_id: str) -> CategoryModel:
        self.entry_period_event_id = entry_period_event_id
        return self

    def with_access_period_event_id(self, access_period_event_id: str) -> CategoryModel:
        self.access_period_event_id = access_period_event_id
        return self

    def with_ignore_user_ids(self, ignore_user_ids: List[str]) -> CategoryModel:
        self.ignore_user_ids = ignore_user_ids
        return self

    def with_generation(self, generation: str) -> CategoryModel:
        self.generation = generation
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        category_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}:categoryModel:{categoryName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            categoryName=category_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModel:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModel:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModel:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_category_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+):categoryModel:(?P<categoryName>.+)', grn)
        if match is None:
            return None
        return match.group('category_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CategoryModel]:
        if data is None:
            return None
        return CategoryModel()\
            .with_category_model_id(data.get('categoryModelId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_minimum_value(data.get('minimumValue'))\
            .with_maximum_value(data.get('maximumValue'))\
            .with_order_direction(data.get('orderDirection'))\
            .with_scope(data.get('scope'))\
            .with_unique_by_user_id(data.get('uniqueByUserId'))\
            .with_sum(data.get('sum'))\
            .with_calculate_fixed_timing_hour(data.get('calculateFixedTimingHour'))\
            .with_calculate_fixed_timing_minute(data.get('calculateFixedTimingMinute'))\
            .with_calculate_interval_minutes(data.get('calculateIntervalMinutes'))\
            .with_additional_scopes([
                Scope.from_dict(data.get('additionalScopes')[i])
                for i in range(len(data.get('additionalScopes')) if data.get('additionalScopes') else 0)
            ])\
            .with_entry_period_event_id(data.get('entryPeriodEventId'))\
            .with_access_period_event_id(data.get('accessPeriodEventId'))\
            .with_ignore_user_ids([
                data.get('ignoreUserIds')[i]
                for i in range(len(data.get('ignoreUserIds')) if data.get('ignoreUserIds') else 0)
            ])\
            .with_generation(data.get('generation'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "categoryModelId": self.category_model_id,
            "name": self.name,
            "metadata": self.metadata,
            "minimumValue": self.minimum_value,
            "maximumValue": self.maximum_value,
            "orderDirection": self.order_direction,
            "scope": self.scope,
            "uniqueByUserId": self.unique_by_user_id,
            "sum": self.sum,
            "calculateFixedTimingHour": self.calculate_fixed_timing_hour,
            "calculateFixedTimingMinute": self.calculate_fixed_timing_minute,
            "calculateIntervalMinutes": self.calculate_interval_minutes,
            "additionalScopes": [
                self.additional_scopes[i].to_dict() if self.additional_scopes[i] else None
                for i in range(len(self.additional_scopes) if self.additional_scopes else 0)
            ],
            "entryPeriodEventId": self.entry_period_event_id,
            "accessPeriodEventId": self.access_period_event_id,
            "ignoreUserIds": [
                self.ignore_user_ids[i]
                for i in range(len(self.ignore_user_ids) if self.ignore_user_ids else 0)
            ],
            "generation": self.generation,
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    last_calculated_ats: List[CalculatedAt] = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_last_calculated_ats(self, last_calculated_ats: List[CalculatedAt]) -> Namespace:
        self.last_calculated_ats = last_calculated_ats
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:ranking:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):ranking:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_last_calculated_ats([
                CalculatedAt.from_dict(data.get('lastCalculatedAts')[i])
                for i in range(len(data.get('lastCalculatedAts')) if data.get('lastCalculatedAts') else 0)
            ])\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "lastCalculatedAts": [
                self.last_calculated_ats[i].to_dict() if self.last_calculated_ats[i] else None
                for i in range(len(self.last_calculated_ats) if self.last_calculated_ats else 0)
            ],
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class ScriptSetting(core.Gs2Model):
    trigger_script_id: str = None
    done_trigger_target_type: str = None
    done_trigger_script_id: str = None
    done_trigger_queue_namespace_id: str = None

    def with_trigger_script_id(self, trigger_script_id: str) -> ScriptSetting:
        self.trigger_script_id = trigger_script_id
        return self

    def with_done_trigger_target_type(self, done_trigger_target_type: str) -> ScriptSetting:
        self.done_trigger_target_type = done_trigger_target_type
        return self

    def with_done_trigger_script_id(self, done_trigger_script_id: str) -> ScriptSetting:
        self.done_trigger_script_id = done_trigger_script_id
        return self

    def with_done_trigger_queue_namespace_id(self, done_trigger_queue_namespace_id: str) -> ScriptSetting:
        self.done_trigger_queue_namespace_id = done_trigger_queue_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[ScriptSetting]:
        if data is None:
            return None
        return ScriptSetting()\
            .with_trigger_script_id(data.get('triggerScriptId'))\
            .with_done_trigger_target_type(data.get('doneTriggerTargetType'))\
            .with_done_trigger_script_id(data.get('doneTriggerScriptId'))\
            .with_done_trigger_queue_namespace_id(data.get('doneTriggerQueueNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "triggerScriptId": self.trigger_script_id,
            "doneTriggerTargetType": self.done_trigger_target_type,
            "doneTriggerScriptId": self.done_trigger_script_id,
            "doneTriggerQueueNamespaceId": self.done_trigger_queue_namespace_id,
        }


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class GitHubCheckoutSetting(core.Gs2Model):
    api_key_id: str = None
    repository_name: str = None
    source_path: str = None
    reference_type: str = None
    commit_hash: str = None
    branch_name: str = None
    tag_name: str = None

    def with_api_key_id(self, api_key_id: str) -> GitHubCheckoutSetting:
        self.api_key_id = api_key_id
        return self

    def with_repository_name(self, repository_name: str) -> GitHubCheckoutSetting:
        self.repository_name = repository_name
        return self

    def with_source_path(self, source_path: str) -> GitHubCheckoutSetting:
        self.source_path = source_path
        return self

    def with_reference_type(self, reference_type: str) -> GitHubCheckoutSetting:
        self.reference_type = reference_type
        return self

    def with_commit_hash(self, commit_hash: str) -> GitHubCheckoutSetting:
        self.commit_hash = commit_hash
        return self

    def with_branch_name(self, branch_name: str) -> GitHubCheckoutSetting:
        self.branch_name = branch_name
        return self

    def with_tag_name(self, tag_name: str) -> GitHubCheckoutSetting:
        self.tag_name = tag_name
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GitHubCheckoutSetting]:
        if data is None:
            return None
        return GitHubCheckoutSetting()\
            .with_api_key_id(data.get('apiKeyId'))\
            .with_repository_name(data.get('repositoryName'))\
            .with_source_path(data.get('sourcePath'))\
            .with_reference_type(data.get('referenceType'))\
            .with_commit_hash(data.get('commitHash'))\
            .with_branch_name(data.get('branchName'))\
            .with_tag_name(data.get('tagName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "apiKeyId": self.api_key_id,
            "repositoryName": self.repository_name,
            "sourcePath": self.source_path,
            "referenceType": self.reference_type,
            "commitHash": self.commit_hash,
            "branchName": self.branch_name,
            "tagName": self.tag_name,
        }


class Stamina(core.Gs2Model):
    stamina_id: str = None
    stamina_name: str = None
    user_id: str = None
    value: int = None
    max_value: int = None
    recover_interval_minutes: int = None
    recover_value: int = None
    overflow_value: int = None
    next_recover_at: int = None
    last_recovered_at: int = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_stamina_id(self, stamina_id: str) -> Stamina:
        self.stamina_id = stamina_id
        return self

    def with_stamina_name(self, stamina_name: str) -> Stamina:
        self.stamina_name = stamina_name
        return self

    def with_user_id(self, user_id: str) -> Stamina:
        self.user_id = user_id
        return self

    def with_value(self, value: int) -> Stamina:
        self.value = value
        return self

    def with_max_value(self, max_value: int) -> Stamina:
        self.max_value = max_value
        return self

    def with_recover_interval_minutes(self, recover_interval_minutes: int) -> Stamina:
        self.recover_interval_minutes = recover_interval_minutes
        return self

    def with_recover_value(self, recover_value: int) -> Stamina:
        self.recover_value = recover_value
        return self

    def with_overflow_value(self, overflow_value: int) -> Stamina:
        self.overflow_value = overflow_value
        return self

    def with_next_recover_at(self, next_recover_at: int) -> Stamina:
        self.next_recover_at = next_recover_at
        return self

    def with_last_recovered_at(self, last_recovered_at: int) -> Stamina:
        self.last_recovered_at = last_recovered_at
        return self

    def with_created_at(self, created_at: int) -> Stamina:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Stamina:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Stamina:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        stamina_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:user:{userId}:stamina:{staminaName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            staminaName=stamina_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):user:(?P<userId>.+):stamina:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):user:(?P<userId>.+):stamina:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):user:(?P<userId>.+):stamina:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):user:(?P<userId>.+):stamina:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_stamina_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):user:(?P<userId>.+):stamina:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('stamina_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Stamina]:
        if data is None:
            return None
        return Stamina()\
            .with_stamina_id(data.get('staminaId'))\
            .with_stamina_name(data.get('staminaName'))\
            .with_user_id(data.get('userId'))\
            .with_value(data.get('value'))\
            .with_max_value(data.get('maxValue'))\
            .with_recover_interval_minutes(data.get('recoverIntervalMinutes'))\
            .with_recover_value(data.get('recoverValue'))\
            .with_overflow_value(data.get('overflowValue'))\
            .with_next_recover_at(data.get('nextRecoverAt'))\
            .with_last_recovered_at(data.get('lastRecoveredAt'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "staminaId": self.stamina_id,
            "staminaName": self.stamina_name,
            "userId": self.user_id,
            "value": self.value,
            "maxValue": self.max_value,
            "recoverIntervalMinutes": self.recover_interval_minutes,
            "recoverValue": self.recover_value,
            "overflowValue": self.overflow_value,
            "nextRecoverAt": self.next_recover_at,
            "lastRecoveredAt": self.last_recovered_at,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class RecoverValueTable(core.Gs2Model):
    name: str = None
    metadata: str = None
    experience_model_id: str = None
    values: List[int] = None

    def with_name(self, name: str) -> RecoverValueTable:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RecoverValueTable:
        self.metadata = metadata
        return self

    def with_experience_model_id(self, experience_model_id: str) -> RecoverValueTable:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> RecoverValueTable:
        self.values = values
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RecoverValueTable]:
        if data is None:
            return None
        return RecoverValueTable()\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "metadata": self.metadata,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
        }


class RecoverIntervalTable(core.Gs2Model):
    name: str = None
    metadata: str = None
    experience_model_id: str = None
    values: List[int] = None

    def with_name(self, name: str) -> RecoverIntervalTable:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RecoverIntervalTable:
        self.metadata = metadata
        return self

    def with_experience_model_id(self, experience_model_id: str) -> RecoverIntervalTable:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> RecoverIntervalTable:
        self.values = values
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RecoverIntervalTable]:
        if data is None:
            return None
        return RecoverIntervalTable()\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "metadata": self.metadata,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
        }


class MaxStaminaTable(core.Gs2Model):
    name: str = None
    metadata: str = None
    experience_model_id: str = None
    values: List[int] = None

    def with_name(self, name: str) -> MaxStaminaTable:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> MaxStaminaTable:
        self.metadata = metadata
        return self

    def with_experience_model_id(self, experience_model_id: str) -> MaxStaminaTable:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> MaxStaminaTable:
        self.values = values
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[MaxStaminaTable]:
        if data is None:
            return None
        return MaxStaminaTable()\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "metadata": self.metadata,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
        }


class StaminaModel(core.Gs2Model):
    stamina_model_id: str = None
    name: str = None
    metadata: str = None
    recover_interval_minutes: int = None
    recover_value: int = None
    initial_capacity: int = None
    is_overflow: bool = None
    max_capacity: int = None
    max_stamina_table: MaxStaminaTable = None
    recover_interval_table: RecoverIntervalTable = None
    recover_value_table: RecoverValueTable = None

    def with_stamina_model_id(self, stamina_model_id: str) -> StaminaModel:
        self.stamina_model_id = stamina_model_id
        return self

    def with_name(self, name: str) -> StaminaModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> StaminaModel:
        self.metadata = metadata
        return self

    def with_recover_interval_minutes(self, recover_interval_minutes: int) -> StaminaModel:
        self.recover_interval_minutes = recover_interval_minutes
        return self

    def with_recover_value(self, recover_value: int) -> StaminaModel:
        self.recover_value = recover_value
        return self

    def with_initial_capacity(self, initial_capacity: int) -> StaminaModel:
        self.initial_capacity = initial_capacity
        return self

    def with_is_overflow(self, is_overflow: bool) -> StaminaModel:
        self.is_overflow = is_overflow
        return self

    def with_max_capacity(self, max_capacity: int) -> StaminaModel:
        self.max_capacity = max_capacity
        return self

    def with_max_stamina_table(self, max_stamina_table: MaxStaminaTable) -> StaminaModel:
        self.max_stamina_table = max_stamina_table
        return self

    def with_recover_interval_table(self, recover_interval_table: RecoverIntervalTable) -> StaminaModel:
        self.recover_interval_table = recover_interval_table
        return self

    def with_recover_value_table(self, recover_value_table: RecoverValueTable) -> StaminaModel:
        self.recover_value_table = recover_value_table
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        stamina_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:model:{staminaName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            staminaName=stamina_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_stamina_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('stamina_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[StaminaModel]:
        if data is None:
            return None
        return StaminaModel()\
            .with_stamina_model_id(data.get('staminaModelId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_recover_interval_minutes(data.get('recoverIntervalMinutes'))\
            .with_recover_value(data.get('recoverValue'))\
            .with_initial_capacity(data.get('initialCapacity'))\
            .with_is_overflow(data.get('isOverflow'))\
            .with_max_capacity(data.get('maxCapacity'))\
            .with_max_stamina_table(MaxStaminaTable.from_dict(data.get('maxStaminaTable')))\
            .with_recover_interval_table(RecoverIntervalTable.from_dict(data.get('recoverIntervalTable')))\
            .with_recover_value_table(RecoverValueTable.from_dict(data.get('recoverValueTable')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "staminaModelId": self.stamina_model_id,
            "name": self.name,
            "metadata": self.metadata,
            "recoverIntervalMinutes": self.recover_interval_minutes,
            "recoverValue": self.recover_value,
            "initialCapacity": self.initial_capacity,
            "isOverflow": self.is_overflow,
            "maxCapacity": self.max_capacity,
            "maxStaminaTable": self.max_stamina_table.to_dict() if self.max_stamina_table else None,
            "recoverIntervalTable": self.recover_interval_table.to_dict() if self.recover_interval_table else None,
            "recoverValueTable": self.recover_value_table.to_dict() if self.recover_value_table else None,
        }


class CurrentStaminaMaster(core.Gs2Model):
    namespace_id: str = None
    settings: str = None

    def with_namespace_id(self, namespace_id: str) -> CurrentStaminaMaster:
        self.namespace_id = namespace_id
        return self

    def with_settings(self, settings: str) -> CurrentStaminaMaster:
        self.settings = settings
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CurrentStaminaMaster]:
        if data is None:
            return None
        return CurrentStaminaMaster()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_settings(data.get('settings'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "settings": self.settings,
        }


class RecoverValueTableMaster(core.Gs2Model):
    recover_value_table_id: str = None
    name: str = None
    metadata: str = None
    description: str = None
    experience_model_id: str = None
    values: List[int] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_recover_value_table_id(self, recover_value_table_id: str) -> RecoverValueTableMaster:
        self.recover_value_table_id = recover_value_table_id
        return self

    def with_name(self, name: str) -> RecoverValueTableMaster:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RecoverValueTableMaster:
        self.metadata = metadata
        return self

    def with_description(self, description: str) -> RecoverValueTableMaster:
        self.description = description
        return self

    def with_experience_model_id(self, experience_model_id: str) -> RecoverValueTableMaster:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> RecoverValueTableMaster:
        self.values = values
        return self

    def with_created_at(self, created_at: int) -> RecoverValueTableMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> RecoverValueTableMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> RecoverValueTableMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        recover_value_table_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:recoverValueTable:{recoverValueTableName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            recoverValueTableName=recover_value_table_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverValueTable:(?P<recoverValueTableName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverValueTable:(?P<recoverValueTableName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverValueTable:(?P<recoverValueTableName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_recover_value_table_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverValueTable:(?P<recoverValueTableName>.+)', grn)
        if match is None:
            return None
        return match.group('recover_value_table_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RecoverValueTableMaster]:
        if data is None:
            return None
        return RecoverValueTableMaster()\
            .with_recover_value_table_id(data.get('recoverValueTableId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_description(data.get('description'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "recoverValueTableId": self.recover_value_table_id,
            "name": self.name,
            "metadata": self.metadata,
            "description": self.description,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class RecoverIntervalTableMaster(core.Gs2Model):
    recover_interval_table_id: str = None
    name: str = None
    metadata: str = None
    description: str = None
    experience_model_id: str = None
    values: List[int] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_recover_interval_table_id(self, recover_interval_table_id: str) -> RecoverIntervalTableMaster:
        self.recover_interval_table_id = recover_interval_table_id
        return self

    def with_name(self, name: str) -> RecoverIntervalTableMaster:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RecoverIntervalTableMaster:
        self.metadata = metadata
        return self

    def with_description(self, description: str) -> RecoverIntervalTableMaster:
        self.description = description
        return self

    def with_experience_model_id(self, experience_model_id: str) -> RecoverIntervalTableMaster:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> RecoverIntervalTableMaster:
        self.values = values
        return self

    def with_created_at(self, created_at: int) -> RecoverIntervalTableMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> RecoverIntervalTableMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> RecoverIntervalTableMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        recover_interval_table_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:recoverIntervalTable:{recoverIntervalTableName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            recoverIntervalTableName=recover_interval_table_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverIntervalTable:(?P<recoverIntervalTableName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverIntervalTable:(?P<recoverIntervalTableName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverIntervalTable:(?P<recoverIntervalTableName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_recover_interval_table_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):recoverIntervalTable:(?P<recoverIntervalTableName>.+)', grn)
        if match is None:
            return None
        return match.group('recover_interval_table_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RecoverIntervalTableMaster]:
        if data is None:
            return None
        return RecoverIntervalTableMaster()\
            .with_recover_interval_table_id(data.get('recoverIntervalTableId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_description(data.get('description'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "recoverIntervalTableId": self.recover_interval_table_id,
            "name": self.name,
            "metadata": self.metadata,
            "description": self.description,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class MaxStaminaTableMaster(core.Gs2Model):
    max_stamina_table_id: str = None
    name: str = None
    metadata: str = None
    description: str = None
    experience_model_id: str = None
    values: List[int] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_max_stamina_table_id(self, max_stamina_table_id: str) -> MaxStaminaTableMaster:
        self.max_stamina_table_id = max_stamina_table_id
        return self

    def with_name(self, name: str) -> MaxStaminaTableMaster:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> MaxStaminaTableMaster:
        self.metadata = metadata
        return self

    def with_description(self, description: str) -> MaxStaminaTableMaster:
        self.description = description
        return self

    def with_experience_model_id(self, experience_model_id: str) -> MaxStaminaTableMaster:
        self.experience_model_id = experience_model_id
        return self

    def with_values(self, values: List[int]) -> MaxStaminaTableMaster:
        self.values = values
        return self

    def with_created_at(self, created_at: int) -> MaxStaminaTableMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> MaxStaminaTableMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> MaxStaminaTableMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        max_stamina_table_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:maxStaminaTable:{maxStaminaTableName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            maxStaminaTableName=max_stamina_table_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):maxStaminaTable:(?P<maxStaminaTableName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):maxStaminaTable:(?P<maxStaminaTableName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):maxStaminaTable:(?P<maxStaminaTableName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_max_stamina_table_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):maxStaminaTable:(?P<maxStaminaTableName>.+)', grn)
        if match is None:
            return None
        return match.group('max_stamina_table_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[MaxStaminaTableMaster]:
        if data is None:
            return None
        return MaxStaminaTableMaster()\
            .with_max_stamina_table_id(data.get('maxStaminaTableId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_description(data.get('description'))\
            .with_experience_model_id(data.get('experienceModelId'))\
            .with_values([
                data.get('values')[i]
                for i in range(len(data.get('values')) if data.get('values') else 0)
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "maxStaminaTableId": self.max_stamina_table_id,
            "name": self.name,
            "metadata": self.metadata,
            "description": self.description,
            "experienceModelId": self.experience_model_id,
            "values": [
                self.values[i]
                for i in range(len(self.values) if self.values else 0)
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class StaminaModelMaster(core.Gs2Model):
    stamina_model_id: str = None
    name: str = None
    metadata: str = None
    description: str = None
    recover_interval_minutes: int = None
    recover_value: int = None
    initial_capacity: int = None
    is_overflow: bool = None
    max_capacity: int = None
    max_stamina_table_name: str = None
    recover_interval_table_name: str = None
    recover_value_table_name: str = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_stamina_model_id(self, stamina_model_id: str) -> StaminaModelMaster:
        self.stamina_model_id = stamina_model_id
        return self

    def with_name(self, name: str) -> StaminaModelMaster:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> StaminaModelMaster:
        self.metadata = metadata
        return self

    def with_description(self, description: str) -> StaminaModelMaster:
        self.description = description
        return self

    def with_recover_interval_minutes(self, recover_interval_minutes: int) -> StaminaModelMaster:
        self.recover_interval_minutes = recover_interval_minutes
        return self

    def with_recover_value(self, recover_value: int) -> StaminaModelMaster:
        self.recover_value = recover_value
        return self

    def with_initial_capacity(self, initial_capacity: int) -> StaminaModelMaster:
        self.initial_capacity = initial_capacity
        return self

    def with_is_overflow(self, is_overflow: bool) -> StaminaModelMaster:
        self.is_overflow = is_overflow
        return self

    def with_max_capacity(self, max_capacity: int) -> StaminaModelMaster:
        self.max_capacity = max_capacity
        return self

    def with_max_stamina_table_name(self, max_stamina_table_name: str) -> StaminaModelMaster:
        self.max_stamina_table_name = max_stamina_table_name
        return self

    def with_recover_interval_table_name(self, recover_interval_table_name: str) -> StaminaModelMaster:
        self.recover_interval_table_name = recover_interval_table_name
        return self

    def with_recover_value_table_name(self, recover_value_table_name: str) -> StaminaModelMaster:
        self.recover_value_table_name = recover_value_table_name
        return self

    def with_created_at(self, created_at: int) -> StaminaModelMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> StaminaModelMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> StaminaModelMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        stamina_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}:model:{staminaName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            staminaName=stamina_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_stamina_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+):model:(?P<staminaName>.+)', grn)
        if match is None:
            return None
        return match.group('stamina_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[StaminaModelMaster]:
        if data is None:
            return None
        return StaminaModelMaster()\
            .with_stamina_model_id(data.get('staminaModelId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_description(data.get('description'))\
            .with_recover_interval_minutes(data.get('recoverIntervalMinutes'))\
            .with_recover_value(data.get('recoverValue'))\
            .with_initial_capacity(data.get('initialCapacity'))\
            .with_is_overflow(data.get('isOverflow'))\
            .with_max_capacity(data.get('maxCapacity'))\
            .with_max_stamina_table_name(data.get('maxStaminaTableName'))\
            .with_recover_interval_table_name(data.get('recoverIntervalTableName'))\
            .with_recover_value_table_name(data.get('recoverValueTableName'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "staminaModelId": self.stamina_model_id,
            "name": self.name,
            "metadata": self.metadata,
            "description": self.description,
            "recoverIntervalMinutes": self.recover_interval_minutes,
            "recoverValue": self.recover_value,
            "initialCapacity": self.initial_capacity,
            "isOverflow": self.is_overflow,
            "maxCapacity": self.max_capacity,
            "maxStaminaTableName": self.max_stamina_table_name,
            "recoverIntervalTableName": self.recover_interval_table_name,
            "recoverValueTableName": self.recover_value_table_name,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    overflow_trigger_script: ScriptSetting = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_overflow_trigger_script(self, overflow_trigger_script: ScriptSetting) -> Namespace:
        self.overflow_trigger_script = overflow_trigger_script
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:stamina:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):stamina:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_overflow_trigger_script(ScriptSetting.from_dict(data.get('overflowTriggerScript')))\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "overflowTriggerScript": self.overflow_trigger_script.to_dict() if self.overflow_trigger_script else None,
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }